#include "ads1115.h"
#include "common.h"

int _sda_pin =0;
int _scl_pin =1; 

#define ADS1115_ADDR 0x48

// I2C写寄存器
static void ads1115_write_register(uint8_t reg, uint16_t val) {
    i2c_start(_sda_pin, _scl_pin);
    i2c_write_byte(_sda_pin, _scl_pin, (ADS1115_ADDR << 1) | 0);
    i2c_write_byte(_sda_pin, _scl_pin, reg);
    i2c_write_byte(_sda_pin, _scl_pin, (val >> 8) & 0xFF);
    i2c_write_byte(_sda_pin, _scl_pin, val & 0xFF);
    i2c_stop(_sda_pin, _scl_pin);
}

// 读取转换寄存器
static uint16_t ads1115_read_conversion() {
    uint16_t result = 0;
    i2c_start(_sda_pin, _scl_pin);
    i2c_write_byte(_sda_pin, _scl_pin, (ADS1115_ADDR << 1) | 0);
    i2c_write_byte(_sda_pin, _scl_pin, 0x00);
    i2c_start(_sda_pin, _scl_pin);
    i2c_write_byte(_sda_pin, _scl_pin, (ADS1115_ADDR << 1) | 1);
    uint8_t msb = i2c_read_byte(_sda_pin, _scl_pin, 1);
    uint8_t lsb = i2c_read_byte(_sda_pin, _scl_pin, 0);
    i2c_stop(_sda_pin, _scl_pin);
    result = (msb << 8) | lsb;
    return result;
}

bool ads1115_init(int _sda_pin, int _scl_pin) {

    pinMode(_sda_pin, OUTPUT);
    pinMode(_scl_pin, OUTPUT);
    digitalWrite(_sda_pin, HIGH);
    digitalWrite(_scl_pin, HIGH);

    return true;
}

// 读取某个单端通道电压，channel = 0~3
float ads1115_read_channel(int channel) {
    uint16_t config = 0;
    if (channel == 0)       config = 0xC203; // MUX=100 (AIN0-GND)
    else if (channel == 1)  config = 0xD203; // MUX=101 (AIN1-GND)
    else if (channel == 2)  config = 0xE203; // MUX=110 (AIN2-GND)
    else if (channel == 3)  config = 0xF203; // MUX=111 (AIN3-GND)
    else return -1.0f;

    ads1115_write_register(0x01, config);

    delayMicroseconds(10000); // 等待转换

    uint16_t raw = ads1115_read_conversion();
    int16_t val = (int16_t)raw;

    float voltage_divided = val * 4.096f / 32768.0f;
    //float real_voltage = val * 6.144f / 32768.0f;
    // 假设分压电阻 R1=2k, R2=10k
    float real_voltage = voltage_divided * (10.0f + 2.0f) / 10.0f;//无分压电阻

    return real_voltage;
}
